@@ -33,7 +33,8 @@ import os
 import sys
 
 from OpenSSL import SSL
-from twisted.internet import ssl
+from twisted.internet import error, ssl
+from twisted.python import log
 
 if sys.platform == "win32":
     # diable pylint warning, as it may be the wrong platform
@@ -58,18 +59,52 @@ else:
     ssl_cert_location = '/etc/ssl/certs'
 
 
-def get_ssl_context(no_verify):
-    """ Get the ssl context """
+class HostnameVerifyContextFactory(ssl.CertificateOptions):
+    """Does hostname checks in addition to certificate checks."""
+
+    def __init__(self, hostname, *args, **kwargs):
+        """Initialize this instance."""
+        super(HostnameVerifyContextFactory, self).__init__(*args, **kwargs)
+        self.expected_hostname = hostname
+
+    def verify_server_hostname(self, conn, cert, errno, depth, preverifyOK):
+        """Verify the server hostname."""
+        if depth == 0:
+            # No extra checks because U1 certs have the right commonName
+            if self.expected_hostname != cert.get_subject().commonName:
+                log.err("Host name does not match certificate. "
+                        "Expected %s but got %s." % (self.expected_hostname,
+                            cert.get_subject().commonName))
+                return False
+        return preverifyOK
+
+    def getContext(self):
+        """The context returned will verify the hostname too."""
+        ctx = super(HostnameVerifyContextFactory, self).getContext()
+        flags = SSL.VERIFY_PEER | SSL.VERIFY_FAIL_IF_NO_PEER_CERT
+        ctx.set_verify(flags, self.verify_server_hostname)
+        return ctx
+
+
+def get_certificates():
+    """Get a list of certificate paths."""
+    ca_file = ssl.Certificate.loadPEM(file(os.path.join(ssl_cert_location,
+                     'UbuntuOne-Go_Daddy_Class_2_CA.pem'), 'r').read())
+    ca_file_2 = ssl.Certificate.loadPEM(file(os.path.join(ssl_cert_location,
+                    'UbuntuOne-Go_Daddy_CA.pem'), 'r').read())
+    ca_file_3 = ssl.Certificate.loadPEM(file(os.path.join(ssl_cert_location,
+                    'ValiCert_Class_2_VA.pem'), 'r').read())
+    return [ca_file.original, ca_file_2.original, ca_file_3.original]
+
+
+def get_ssl_context(no_verify, hostname=None):
+    """Get the ssl context."""
     if no_verify:
         ctx = ssl.ClientContextFactory()
     else:
-        ca_file = ssl.Certificate.loadPEM(file(
-            os.path.join(ssl_cert_location,
-                         'UbuntuOne-Go_Daddy_Class_2_CA.pem'), 'r').read())
-        ca_file_2 = ssl.Certificate.loadPEM(file(
-            os.path.join(ssl_cert_location,
-                        'UbuntuOne-Go_Daddy_CA.pem'), 'r').read())
-        ctx = ssl.CertificateOptions(verify=True,
-            caCerts=[ca_file.original, ca_file_2.original],
-                     method=SSL.SSLv23_METHOD)
+        if hostname is None:
+            raise error.CertificateError(
+                'No hostname specified. Unable to verify SSL certificate.')
+        ctx = HostnameVerifyContextFactory(hostname, verify=True,
+                          caCerts=get_certificates(), method=SSL.SSLv23_METHOD)
     return ctx
